@@ -83,14 +83,13 @@ KigDocument::KigDocument( QWidget *parentWidget, const char *,
 			  const QStringList& )
   : KParts::ReadWritePart( parent, name ),
     mMode( 0 ),
-    numViews(0),
     mcoordsystem( new EuclideanCoords )
 {
   // we need an instance
   setInstance( KigDocumentFactory::instance() );
 
   // we need a widget, to actually show the document
-  m_widget = new KigView(this, parentWidget, "kig_view");
+  m_widget = new KigView(this, false, parentWidget, "kig_view");
   // notify the part that this is our internal widget
   setWidget( m_widget );
 
@@ -159,19 +158,13 @@ void KigDocument::setupActions()
     actionCollection(), "types_edit");
   aConfigureTypes->setToolTip(i18n("Manage macro types."));
 
-//   tmp = l->loadIcon( "window_fullscreen", KIcon::User );
-//   aFullScreen = new KAction(
-//     i18n( "Full Screen" ), tmp, 0, this, SLOT( startKiosk() ),
-//     actionCollection(), "view_fullscreen" );
-//   aFullScreen->setToolTip( i18n( "View this document full-screen." ) );
-
   tmp = l->loadIcon( "pointxy", KIcon::User );
   aFixedPoint = new AddFixedPointAction( this, tmp, actionCollection() );
 
   tmp = l->loadIcon( "new", KIcon::User );
   new TestAction( this, tmp, actionCollection() );
 
-  KigExportManager::instance()->addMenuAction( this, mainWidget()->realWidget(),
+  KigExportManager::instance()->addMenuAction( this, m_widget->realWidget(),
                                                actionCollection() );
 };
 
@@ -208,14 +201,12 @@ KigDocument::~KigDocument()
         file != dataFiles.end(); ++file )
   {
     QFile f( *file );
-    kdDebug() << "removing: " << *file << endl;
     f.remove();
   };
 
   // save our types...
   QString typesDir = KGlobal::dirs()->saveLocation("appdata", "kig-types");
   if (typesDir[typesDir.length() - 1] != '/') typesDir += '/';
-  kdDebug() << k_funcinfo << " : saving types to: " << typesDir << endl;
   MacroList* macrolist = MacroList::instance();
   macrolist->save( macrolist->macros(), typesDir + "macros.kigt" );
 
@@ -241,8 +232,6 @@ KigDocument::~KigDocument()
 
 bool KigDocument::openFile()
 {
-  kdDebug() << k_funcinfo << m_file << endl;
-
   // m_file is always local, so we can use findByPath instead of
   // findByURL...
   KMimeType::Ptr mimeType = KMimeType::findByPath ( m_file );
@@ -255,8 +244,8 @@ bool KigDocument::openFile()
     KMessageBox::sorry
       (
         widget(),
-        i18n( "You tried to open a document of type \"%1\".  Unfortunately, "
-              "Kig doesn't support this format.  If you think the format in "
+        i18n( "You tried to open a document of type \"%1\". Unfortunately, "
+              "Kig doesn't support this format. If you think the format in "
               "question would be worth implementing support for, you can "
               "always ask me nicely on mailto:devriese@kde.org "
               "or do the work yourself and send me a patch."
@@ -265,19 +254,8 @@ bool KigDocument::openFile()
         );
     return false;
   };
-  if ( filter->load (m_file, *this) != KigFilter::OK )
-  {
-    KMessageBox::sorry( widget(), i18n(
-        "The file you tried to open contains some elements that Kig currently "
-        "doesn't understand.  It is possible that the file somehow got "
-        "corrupted and is no longer usable.  If you know that the file is "
-        "valid, and you think Kig should be able to open it, you can try to "
-        "send me a copy of the file and ask me to check it out.  If you want "
-        "more certain results, you can always do the work yourself ( since Kig "
-        "is Free Software ), and send me a patch."
-                            ) );
+  if ( !filter->load (m_file, *this) )
     return false;
-  };
 
   setModified(false);
   mhistory->clear();
@@ -286,7 +264,7 @@ bool KigDocument::openFile()
   tmp.calc( *this );
   emit recenterScreen();
 
-  mainWidget()->realWidget()->redrawScreen();
+  mode()->redrawScreen();
 
   return true;
 }
@@ -296,75 +274,41 @@ bool KigDocument::saveFile()
   if ( m_file.isEmpty() ) return internalSaveAs();
   // mimetype:
   KMimeType::Ptr mimeType = KMimeType::findByPath ( m_file );
-  kdDebug() << k_funcinfo << "mimetype: " << mimeType->name() << endl;
-  // filter...
-  KigFilter* filter = KigFilters::instance()->find( mimeType->name() );
-  if ( !filter )
+  if ( mimeType->name() != "application/x-kig" )
   {
     // we don't support this mime type...
     KMessageBox::sorry
       (
         widget(),
-        i18n( "You tried to save to a file of type \"%1\".  Unfortunately, "
-              "Kig doesn't support this format.  If you think the format in "
-              "question would be worth implementing support for, you can "
-              "always ask me nicely on mailto:devriese@kde.org "
-              "or do the work yourself and send me a patch."
-          ).arg(mimeType->name()),
+        i18n( "Kig does not support saving to any other file format than "
+              "its own." ).arg( mimeType->name() ),
         i18n( "Format not Supported" )
         );
     return false;
   };
 
-  KigFilter::Result result = filter->save( *this, m_file );
-  if ( result == KigFilter::OK )
+  if ( KigFilters::instance()->save( *this, m_file ) )
   {
     setModified ( false );
     mhistory->documentSaved();
     return true;
   }
-  else if ( result == KigFilter::FileNotFound )
-  {
-    // i know i need to change the enum value name, but this means
-    // that the file could not be opened...
-    KMessageBox::sorry( m_widget,
-                        i18n( "The file \"%1\" could not be opened.  Please "
-                              "check if the file permissions are set correctly." )
-                        .arg( m_file ) );
-    return false;
-
-  }
-  else // if ( result == KigFilter::NotSupported )
-  {
-    // we don't support this mime type...
-    KMessageBox::sorry
-      (
-        widget(),
-        i18n( "You tried to save to a file of MIME type \"%1\".  Unfortunately, "
-              "Kig doesn't support this format.  If you think the format in "
-              "question would be worth implementing support for, you can "
-              "always ask me nicely on mailto:devriese@kde.org "
-              "or do the work yourself and send me a patch."
-          ).arg(mimeType->name()),
-        i18n( "Format not Supported" )
-        );
-  };
   return false;
 };
 
 void KigDocument::addObject(Object* o)
 {
-  mhistory->addCommand( new AddObjectsCommand(this, o) );
+  mhistory->addCommand( KigCommand::addCommand( *this, o ) );
 };
 
 void KigDocument::addObjects( const Objects& os )
 {
-  mhistory->addCommand( new AddObjectsCommand(this, os) );
+  mhistory->addCommand( KigCommand::addCommand( *this, os ) );
 }
 
 void KigDocument::_addObject( Object* o )
 {
-  mObjs.push_back( o );
+  mObjs.upush( o );
   setModified(true);
 };
 
@@ -373,9 +317,19 @@ void KigDocument::delObject(Object* o)
   // we delete all children and their children etc. too...
   Objects all = o->getAllChildren();
   all.upush(o);
-  mhistory->addCommand( new RemoveObjectsCommand(this, all) );
+  mhistory->addCommand( KigCommand::removeCommand( *this, all ) );
 };
 
+void KigDocument::_delObjects( const Objects& o )
+{
+  for ( Objects::const_iterator i = o.begin(); i != o.end(); ++i )
+  {
+    mObjs.remove( *i );
+    (*i)->setSelected( false );
+  };
+  setModified( true );
+}
+
 void KigDocument::_delObject(Object* o)
 {
   mObjs.remove( o );
@@ -413,7 +367,6 @@ Objects KigDocument::whatIsInHere( const Rect& p, const KigWidget& w )
 
 Rect KigDocument::suggestedRect() const
 {
-  if( mObjs.empty() ) return Rect( -6., -6., 12., 12. );
   bool rectInited = false;
   Rect r(0.,0.,0.,0.);
   for (Objects::const_iterator i = mObjs.begin(); i != mObjs.end(); ++i )
@@ -429,13 +382,15 @@ Rect KigDocument::suggestedRect() const
         r.setContains( static_cast<const PointImp*>( (*i)->imp() )->coordinate() );
     };
   };
+
+  if ( ! rectInited )
+    return Rect( -5.5, -5.5, 11., 11. );
   r.setContains( Coordinate( 0, 0 ) );
-  Coordinate centre = r.center();
-  r.setWidth(r.width()*2);
   if (r.width() == 0) r.setWidth( 1 );
-  r.setHeight(r.height()*2);
   if (r.height() == 0) r.setHeight( 1 );
-  r.setCenter(centre);
+  Coordinate center = r.center();
+  r *= 2;
+  r.setCenter(center);
   return r;
 }
 
@@ -450,10 +405,11 @@ void KigDocument::setMode( KigMode* m )
   mMode = m;
   m->enableActions();
 }
-void KigDocument::_addObjects( const Objects& o)
+
+void KigDocument::_addObjects( const Objects& os )
 {
-  for( Objects::const_iterator i = o.begin(); i != o.end(); ++i )
-    _addObject( *i );
+  mObjs |= os;
+  setModified( true );
 }
 
 void KigDocument::deleteObjects()
@@ -481,11 +437,6 @@ void KigDocument::editTypes()
   mode()->editTypes();
 }
 
-void KigDocument::startKiosk()
-{
-  mode()->startKiosk();
-}
-
 void KigDocument::setUnmodified()
 {
   setModified( false );
@@ -507,7 +458,7 @@ void KigDocument::delObjects( const Objects& os )
     if ( mObjs.contains( *i ) )
       dos.upush( *i );
   if ( dos.empty() ) return;
-  mhistory->addCommand( new RemoveObjectsCommand( this, dos ) );
+  mhistory->addCommand( KigCommand::removeCommand( *this, dos ) );
 }
 
 void KigDocument::enableConstructActions( bool enabled )
@@ -559,7 +510,7 @@ bool KigDocument::internalSaveAs()
 {
   // this slot is connected to the KStdAction::saveAs action...
   QString formats;
-  formats = QString::fromUtf8("*.kig|Kig Documents (*.kig)");
+  formats = i18n("*.kig|Kig Documents (*.kig)");
 
   //  formats += "\n";
   //  formats += KImageIO::pattern( KImageIO::Writing );
@@ -569,7 +520,7 @@ bool KigDocument::internalSaveAs()
   else if ( QFileInfo( file_name ).exists() )
   {
     int ret = KMessageBox::warningYesNo( m_widget,
-                                         i18n( "The file \"%1\" already exists.  Do you wish to overwrite it?" )
+                                         i18n( "The file \"%1\" already exists. Do you wish to overwrite it?" )
                                          .arg( file_name ) );
     if ( ret != KMessageBox::Yes )
     {
@@ -580,11 +531,6 @@ bool KigDocument::internalSaveAs()
   return true;
 }
 
-KigView* KigDocument::mainWidget()
-{
-  return m_widget;
-}
-
 void KigDocument::runMode( KigMode* m )
 {
   KigMode* prev = mMode;
@@ -677,7 +623,6 @@ void KigDocument::setupMacroTypes()
           file != dataFiles.end(); ++file )
     {
       myvector<Macro*> nmacros;
-      kdDebug() << k_funcinfo << " loading types from: " << *file << endl;
       bool ok = MacroList::instance()->load( *file, nmacros, *this );
       if ( ! ok ) continue;
       copy( nmacros.begin(), nmacros.end(), back_inserter( macros ) );
@@ -720,3 +665,25 @@ void KigDocument::setupBuiltinMacros()
     };
   };
 }
+
+void KigDocument::addWidget( KigWidget* v )
+{
+  mwidgets.push_back( v );
+}
+
+void KigDocument::delWidget( KigWidget* v )
+{
+  std::remove( mwidgets.begin(), mwidgets.end(), v );
+}
+
+const std::vector<KigWidget*>& KigDocument::widgets()
+{
+  return mwidgets;
+}
+
+CoordinateSystem* KigDocument::switchCoordinateSystem( CoordinateSystem* s )
+{
+  CoordinateSystem* ret = mcoordsystem;
+  mcoordsystem = s;
+  return ret;
+}
