@@ -57,9 +57,6 @@ class ScreenInfo;
 class KigDocument : public KParts::ReadWritePart
 {
   Q_OBJECT
-
-  friend class AddObjectsCommand;
-  friend class RemoveObjectsCommand;
 public:
   /**
    * Default constructor
@@ -74,7 +71,7 @@ public:
    */
   virtual ~KigDocument();
 
-  KigView* mainWidget();
+  const std::vector<KigWidget*>& widgets();
 
 /*********************** KPart interface *************************/
 
@@ -92,10 +89,6 @@ protected:
    */
   virtual bool saveFile();
 
-signals:
-  // we control the app's statusbar...
-  void setStatusBarText ( const QString & text );
-
 public:
   void emitStatusBarText( const QString& text );
 
@@ -111,7 +104,6 @@ public slots:
   void showHidden();
   void newMacro();
   void editTypes();
-  void startKiosk();
   // equivalent to setModified( false ); ( did i mention i don't like
   // signals/slots for being this inflexible...
   // this is connected to mhistory->documentRestored();
@@ -119,12 +111,21 @@ public slots:
 
   /****************** cooperation with stuff ******************/
 public:
-  void addView(KigView*) { numViews++; };
-  void delView(KigView*) { numViews--; };
+  void addWidget( KigWidget* );
+  void delWidget( KigWidget* );
 
   const Objects& objects() const { return mObjs;};
   void setObjects( const Objects& os );
   const CoordinateSystem& coordinateSystem() const;
+
+  /**
+   * sets the coordinate system to s, and returns the old one..
+   */
+  CoordinateSystem* switchCoordinateSystem( CoordinateSystem* s );
+
+  /**
+   * sets the coordinate system to s, and deletes the old one..
+   */
   void setCoordinateSystem( CoordinateSystem* s );
   KigMode* mode() { return mMode; };
   void setMode( KigMode* );
@@ -163,6 +164,7 @@ public:
   void _addObject( Object* inObject );
   void _addObjects( const Objects& o);
   void _delObject( Object* inObject );
+  void _delObjects( const Objects& o );
 
 protected:
   void setupActions();
@@ -223,7 +225,7 @@ public:
   void enableConstructActions( bool enabled );
 
 protected:
-  int numViews;
+  std::vector<KigWidget*> mwidgets;
 
   KigView* m_widget;
 
