@@ -30,7 +30,7 @@
 static KCmdLineOptions options[] =
   {
     { "+[URL]", I18N_NOOP( "Document to open." ), 0 },
-    { 0, 0, 0 }
+    KCmdLineLastOption
   };
 
 class KigApplication
