@@ -33,8 +33,6 @@
 #include <knumvalidator.h>
 #include <kmessagebox.h>
 #include <klineeditdlg.h>
-#include <klocale.h>
-#include <kglobal.h>
 
 #include <cmath>
 #include <string>
@@ -49,6 +47,7 @@ using std::log10;
 class CoordinateValidator
   : public QValidator
 {
+  bool mpolar;
 #ifdef KIG_USE_KDOUBLEVALIDATOR
   KDoubleValidator mdv;
 #else
@@ -56,15 +55,17 @@ class CoordinateValidator
 #endif
   mutable QRegExp mre;
 public:
-  CoordinateValidator();
+  CoordinateValidator( bool polar );
   ~CoordinateValidator();
   State validate ( QString & input,  int & pos ) const;
   void fixup ( QString & input ) const;
 };
 
 
-CoordinateValidator::CoordinateValidator()
-  : QValidator( 0, 0 ), mdv( 0, 0 ), mre( "\\(? ?([0-9.,+-]+); ?([0-9.,+-]+) ?\\)?" )
+CoordinateValidator::CoordinateValidator( bool polar )
+  : QValidator( 0, 0 ), mpolar( polar ), mdv( 0, 0 ),
+    mre( polar ? "\\(? ?([0-9.,+-]+); ?([0-9.,+-]+) ?? ?\\)?"
+         : "\\(? ?([0-9.,+-]+); ?([0-9.,+-]+) ?\\)?" )
 {
 }
 
@@ -76,9 +77,14 @@ QValidator::State CoordinateValidator::validate( QString & input, int & pos ) co
 {
   QString tinput = input;
   if ( tinput[tinput.length() - 1 ] == ')' ) tinput.truncate( tinput.length() - 1 );
-  while ( tinput[tinput.length() - 1 ] == ' ' ) tinput.truncate( tinput.length() - 1 );
+  if ( mpolar )
+  {
+    if ( tinput[tinput.length() - 1 ] == ' ' ) tinput.truncate( tinput.length() - 1 );
+    if ( tinput[tinput.length() - 1 ] == '' ) tinput.truncate( tinput.length() - 1 );
+  };
+  if( tinput[tinput.length() - 1 ] == ' ' ) tinput.truncate( tinput.length() - 1 );
   if ( tinput[0] == '(' ) tinput = tinput.mid( 1 );
-  while ( tinput[0] == ' ' ) tinput = tinput.mid( 1 );
+  if( tinput[0] == ' ' ) tinput = tinput.mid( 1 );
   int scp = tinput.find( ';' );
   if ( scp == -1 ) return mdv.validate( tinput, pos ) == Invalid ? Invalid : Valid;
   else
@@ -114,9 +120,13 @@ void CoordinateValidator::fixup( QString & input ) const
   {
     sc = input.length();
     KLocale* l = KGlobal::locale();
-    input.append( QString::fromLatin1( ";" ) + l->positiveSign() +
-                  QString::fromLatin1( "0" ) + l->decimalSymbol() +
-                  QString::fromLatin1( "0" ) );
+    if ( mpolar )
+      input.append( QString::fromLatin1( ";" ) + l->positiveSign() +
+                    QString::fromLatin1( "0" ) );
+    else
+      input.append( QString::fromLatin1( ";" ) + l->positiveSign() +
+                    QString::fromLatin1( "0" ) + l->decimalSymbol() +
+                    QString::fromLatin1( "0" ) );
   };
   mre.exactMatch( input );
   QString ds1 = mre.cap( 1 );
@@ -207,9 +217,9 @@ void EuclideanCoords::drawGrid( KigPainter& p, bool showgrid, bool showaxes ) co
   const double vmin = p.window().bottom();
 
   // the number of intervals we would like to have:
-  // we try to have one of them per 25 pixels or so..
+  // we try to have one of them per 40 pixels or so..
   const int ntick = static_cast<int>(
-    kigMax( hmax - hmin, vmax - vmin ) / p.pixelWidth() / 25. ) + 1;
+    kigMax( hmax - hmin, vmax - vmin ) / p.pixelWidth() / 40. ) + 1;
 
   const double hrange = nicenum( hmax - hmin, false );
   const double vrange = nicenum( vmax - vmin, false );
@@ -217,10 +227,10 @@ void EuclideanCoords::drawGrid( KigPainter& p, bool showgrid, bool showaxes ) co
   const double hd = nicenum( hrange / ( ntick - 1 ), true );
   const double vd = nicenum( vrange / ( ntick - 1 ), true );
 
-  const double hgraphmin = floor( hmin / hd) * hd;
-  const double hgraphmax = ceil( hmax / hd ) * hd;
-  const double vgraphmin = floor( vmin / vd ) * vd;
-  const double vgraphmax = ceil( vmax / vd ) * vd;
+  const double hgraphmin = ceil( hmin / hd) * hd;
+  const double hgraphmax = floor( hmax / hd ) * hd;
+  const double vgraphmin = ceil( vmin / vd ) * vd;
+  const double vgraphmax = floor( vmax / vd ) * vd;
 
   const int hnfrac = max( (int) - floor( log10( hd ) ), 0 );
   const int vnfrac = max( (int) - floor( log10( vd ) ), 0 );
@@ -267,7 +277,7 @@ void EuclideanCoords::drawGrid( KigPainter& p, bool showgrid, bool showaxes ) co
     for ( double i = vgraphmin; i <= vgraphmax + vd/2; i += vd )
     {
       if( fabs( i ) < 1e-8 ) continue;
-      p.drawText ( Rect( Coordinate( 0, i ), hd, vd ).normalized(),
+      p.drawText ( Rect( Coordinate( 0, i ), 2*hd, vd ).normalized(),
                    KGlobal::locale()->formatNumber( i, vnfrac ),
                    AlignBottom | AlignLeft
         );
@@ -328,10 +338,10 @@ QString PolarCoords::fromScreen( const Coordinate& pt, const KigDocument& d ) co
   int l = max( 0, (int) ( 3 - log10( m ) ) );
 
   double r = pt.length();
-  double theta = atan2( pt.y, pt.x );
+  double theta = atan2( pt.y, pt.x ) * 180 / M_PI;
 
   QString rs = KGlobal::locale()->formatNumber( r, l );
-  QString ts = KGlobal::locale()->formatNumber( theta, 2 );
+  QString ts = KGlobal::locale()->formatNumber( theta, 0 );
 
   return QString::fromLatin1("( %1; %2 )").arg( rs ).arg( ts );
 }
@@ -345,7 +355,7 @@ QString PolarCoords::coordinateFormatNotice() const
 
 Coordinate PolarCoords::toScreen(const QString& s, bool& ok) const
 {
-  QRegExp regexp("^([^;]+);(.+)$");
+  QRegExp regexp("\\(? ?([0-9.,+-]+); ?([0-9.,+-]+) ?? ?\\)?" );
   ok = ( regexp.search( s ) == 0 );
   if (ok)
   {
@@ -357,6 +367,8 @@ Coordinate PolarCoords::toScreen(const QString& s, bool& ok) const
     double theta = KGlobal::locale()->readNumber( ts, &ok );
     if ( ! ok ) theta = ts.toDouble( &ok );
     if ( ! ok ) return Coordinate();
+    theta *= M_PI;
+    theta /= 180;
     return Coordinate( cos( theta ) * r, sin( theta ) * r );
   }
   else return Coordinate();
@@ -377,9 +389,9 @@ void PolarCoords::drawGrid( KigPainter& p, bool showgrid, bool showaxes ) const
   const double vmin = sqrt2*p.window().bottom();
 
   // the intervals:
-  // we try to have one of them per 25 pixels or so..
+  // we try to have one of them per 40 pixels or so..
   const int ntick = static_cast<int>(
-    kigMax( hmax - hmin, vmax - vmin ) / p.pixelWidth() / 25 ) + 1;
+    kigMax( hmax - hmin, vmax - vmin ) / p.pixelWidth() / 40 ) + 1;
 
   const double hrange = nicenum( hmax - hmin, false );
   const double vrange = nicenum( vmax - vmin, false );
@@ -468,36 +480,14 @@ void PolarCoords::drawGrid( KigPainter& p, bool showgrid, bool showaxes ) const
   }; // if( showaxes )
 }
 
-static const char* euclideanTypeString = "Euclidean";
-static const char* polarTypeString = "Polar";
-
-CoordinateSystem* CoordinateSystemFactory::build( const char* type )
-{
-  if ( std::string( euclideanTypeString ) == type )
-    return new EuclideanCoords;
-  if ( std::string( polarTypeString ) == type )
-    return new PolarCoords;
-  else return 0;
-}
-
-const char* EuclideanCoords::type() const
-{
-  return euclideanTypeString;
-}
-
-const char* PolarCoords::type() const
-{
-  return polarTypeString;
-}
-
 QValidator* EuclideanCoords::coordinateValidator() const
 {
-  return new CoordinateValidator();
+  return new CoordinateValidator( false );
 }
 
 QValidator* PolarCoords::coordinateValidator() const
 {
-  return new CoordinateValidator();
+  return new CoordinateValidator( true );
 }
 
 const Coordinate CoordinateSystem::getCoordFromUser( const QString& caption,
@@ -521,7 +511,70 @@ const Coordinate CoordinateSystem::getCoordFromUser( const QString& caption,
     ret = toScreen( value, *ok );
     if ( *ok ) done = true;
     else
-      KMessageBox::sorry( parent, i18n( "The coordinate you entered was not valid.  Please try again.") );
+      KMessageBox::sorry( parent, i18n( "The coordinate you entered was not valid. Please try again.") );
   };
   return ret;
 }
+
+QStringList CoordinateSystemFactory::names()
+{
+  QStringList ret;
+  ret << i18n( "&Euclidean" )
+      << i18n( "&Polar" );
+  return ret;
+}
+
+CoordinateSystem* CoordinateSystemFactory::build( int which )
+{
+  if ( which == Euclidean )
+    return new EuclideanCoords;
+  else if ( which == Polar )
+    return new PolarCoords;
+  else return 0;
+}
+
+static const char* euclideanTypeString = "Euclidean";
+static const char* polarTypeString = "Polar";
+
+CoordinateSystem* CoordinateSystemFactory::build( const char* type )
+{
+  if ( std::string( euclideanTypeString ) == type )
+    return new EuclideanCoords;
+  if ( std::string( polarTypeString ) == type )
+    return new PolarCoords;
+  else return 0;
+}
+
+const char* EuclideanCoords::type() const
+{
+  return euclideanTypeString;
+}
+
+const char* PolarCoords::type() const
+{
+  return polarTypeString;
+}
+
+int EuclideanCoords::id() const
+{
+  return CoordinateSystemFactory::Euclidean;
+}
+
+int PolarCoords::id() const
+{
+  return CoordinateSystemFactory::Polar;
+}
+
+QString CoordinateSystemFactory::setCoordinateSystemStatement( int id )
+{
+  switch( id )
+  {
+  case Euclidean:
+    return i18n( "Set Euclidean Coordinate System" );
+  case Polar:
+    return i18n( "Set Polar Coordinate System" );
+  default:
+    assert( false );
+  }
+}
+
