@@ -28,6 +28,7 @@
 #include "../kig/kig_view.h"
 
 #include "../objects/object_type.h"
+#include "../objects/other_type.h"
 #include "../objects/object_imp.h"
 #include "../objects/bogus_imp.h"
 
@@ -72,8 +73,7 @@ void StandardConstructorBase::handleArgs(
   const Objects& os, KigDocument& d,
   KigWidget& v ) const
 {
-  Objects args = margsparser.parse( os );
-  Objects bos = build( args, d, v );
+  Objects bos = build( os, d, v );
   bos.calc( d );
   d.addObjects( bos );
 }
@@ -276,6 +276,7 @@ QString StandardConstructorBase::useText( const Object& o, const Objects& sel,
   using namespace std;
   Args args;
   transform( sel.begin(), sel.end(), back_inserter( args ), mem_fun( &Object::imp ) );
+  args.push_back( o.imp() );
 
   const char* ret = margsparser.usetext( o.imp(), args );
   if ( ! ret ) return QString::null;
@@ -354,6 +355,7 @@ QString MacroConstructor::useText( const Object& o, const Objects& sel,
   Args args;
   transform( sel.begin(), sel.end(), back_inserter( args ),
              mem_fun( &Object::imp ) );
+  args.push_back( o.imp() );
   return mparser.usetext( o.imp(), args );
 }
 
@@ -451,3 +453,52 @@ bool ObjectConstructor::isIntersection() const
   return false;
 }
 
+PropertyObjectConstructor::PropertyObjectConstructor(
+  const int imprequirement, const char* usetext,
+  const char* descname, const char* desc,
+  const char* iconfile, const char* propertyinternalname )
+  : StandardConstructorBase( descname, desc, iconfile, mparser ),
+    mpropinternalname( propertyinternalname )
+{
+  ArgParser::spec argsspec[1];
+  argsspec[0].type = imprequirement;
+  argsspec[0].usetext = usetext;
+  mparser.initialize( argsspec, 1 );
+}
+
+PropertyObjectConstructor::~PropertyObjectConstructor()
+{
+}
+
+void PropertyObjectConstructor::drawprelim(
+  KigPainter& p, const Objects& parents,
+  const KigDocument& d ) const
+{
+  int index = parents[0]->propertiesInternalNames().findIndex( mpropinternalname );
+  assert ( index != -1 );
+  ObjectImp* imp = parents[0]->property( index, d );
+  imp->draw( p );
+  delete imp;
+}
+
+Objects PropertyObjectConstructor::build(
+  const Objects& parents, KigDocument&,
+  KigWidget& ) const
+{
+  int index = parents[0]->propertiesInternalNames().findIndex( mpropinternalname );
+  assert( index != -1 );
+  Objects ret;
+  ret.push_back( new PropertyObject( parents[0], index ) );
+  ret.push_back( new RealObject( CopyObjectType::instance(), ret ) );
+  return ret;
+}
+
+void PropertyObjectConstructor::plug( KigDocument*, KigGUIAction* )
+{
+}
+
+bool PropertyObjectConstructor::isTransform() const
+{
+  return false;
+}
+
