@@ -223,16 +223,6 @@ std::vector<ObjectImp*> ObjectHierarchy::calc( const Args& a, const KigDocument&
   };
 }
 
-ObjectHierarchy::ObjectHierarchy( const Objects& from, const Object* to )
-  : mnumberofargs( from.size() ), mnumberofresults( 1 )
-{
-  margrequirements.resize( from.size(), -1 );
-  std::map<const Object*, int> seenmap;
-  for ( uint i = 0; i < from.size(); ++i )
-    seenmap[from[i]] = i;
-  visit( to, seenmap );
-}
-
 int ObjectHierarchy::visit( const Object* o, std::map<const Object*, int>& seenmap )
 {
   using namespace std;
@@ -266,7 +256,6 @@ int ObjectHierarchy::visit( const Object* o, std::map<const Object*, int>& seenm
     {
       Object* parent = o->parents()[i];
       Objects oparents = o->parents();
-      oparents.remove( parent );
 
       margrequirements[parents[i]] = kMax( margrequirements[parents[i]],
                                            o->impRequirement( parent, oparents ) );
@@ -278,8 +267,10 @@ int ObjectHierarchy::visit( const Object* o, std::map<const Object*, int>& seenm
   {
     assert( parents.size() == 1 );
     int parent = parents.front();
-    Object* op = p[parent];
+    Object* op = p.front();
+    assert( op );
     uint propid = static_cast<const PropertyObject*>( o )->propId();
+    assert( propid < op->propertiesInternalNames().size() );
     mnodes.push_back( new FetchPropertyNode( parent, op->propertiesInternalNames()[propid], propid ) );
   } else assert( false );
   seenmap[o] = mnumberofargs + mnodes.size() - 1;
@@ -318,10 +309,20 @@ ObjectHierarchy ObjectHierarchy::withFixedArgs( const Args& a ) const
   return ret;
 }
 
+ObjectHierarchy::ObjectHierarchy( const Objects& from, const Object* to )
+  : mnumberofargs( from.size() ), mnumberofresults( 1 )
+{
+  margrequirements.resize( from.size(), ObjectImp::ID_AnyImp );
+  std::map<const Object*, int> seenmap;
+  for ( uint i = 0; i < from.size(); ++i )
+    seenmap[from[i]] = i;
+  visit( to, seenmap );
+}
+
 ObjectHierarchy::ObjectHierarchy( const Objects& from, const Objects& to )
   : mnumberofargs( from.size() ), mnumberofresults( to.size() )
 {
-  margrequirements.resize( from.size(), -1 );
+  margrequirements.resize( from.size(), ObjectImp::ID_AnyImp );
   std::map<const Object*, int> seenmap;
   for ( uint i = 0; i < from.size(); ++i )
     seenmap[from[i]] = i;
@@ -400,7 +401,7 @@ ObjectHierarchy::ObjectHierarchy( const QDomElement& parent )
     tmp = e.attribute( "requirement" );
     int req = ObjectImp::stringToID( tmp.latin1() );
     if ( req == -1 ) req = ObjectImp::ID_AnyImp; // sucks, i know..
-    margrequirements.resize( mnumberofargs, -1 );
+    margrequirements.resize( mnumberofargs, ObjectImp::ID_AnyImp );
     margrequirements[id - 1] = req;
   }
   for (; !e.isNull(); e = e.nextSibling().toElement() )
@@ -461,7 +462,6 @@ ArgParser ObjectHierarchy::argParser() const
   for ( uint i = 0; i < margrequirements.size(); ++i )
   {
     int req = margrequirements[i];
-    assert( req != -1 );
     ArgParser::spec spec;
     spec.type = req;
     spec.usetext = ObjectImp::selectStatement( req );
@@ -518,3 +518,21 @@ ObjectHierarchy ObjectHierarchy::transformFinalObject( const Transformation& t )
   ret.mnodes.push_back( new ApplyTypeNode( type, parents ) );
   return ret;
 }
+
+bool operator==( const ObjectHierarchy& lhs, const ObjectHierarchy& rhs )
+{
+  if ( ! ( lhs.mnumberofargs == rhs.mnumberofargs &&
+           lhs.mnumberofresults == rhs.mnumberofresults &&
+           lhs.margrequirements == rhs.margrequirements &&
+           lhs.mnodes.size() == rhs.mnodes.size() ) )
+    return false;
+
+  // this isn't entirely correct, but it will do, because we don't
+  // really want to know whether the hierarchies are different, but
+  // whether rhs has changed with regard to lhs..
+  for ( uint i = 0; i < lhs.mnodes.size(); ++i )
+    if ( lhs.mnodes[i] != lhs.mnodes[i] )
+      return false;
+
+  return true;
+};
