@@ -71,9 +71,6 @@ public:
   ConicConicIntersectionConstructor();
   ~ConicConicIntersectionConstructor();
 
-  QString useText( const Object& o, const Objects& sel, const KigDocument& d,
-                   const KigWidget& v ) const;
-
   void drawprelim( KigPainter& p, const Objects& parents,
                    const KigDocument& ) const;
   Objects build( const Objects& os, KigDocument& d, KigWidget& w ) const;
@@ -88,8 +85,6 @@ class ConicLineIntersectionConstructor
 public:
   ConicLineIntersectionConstructor();
   ~ConicLineIntersectionConstructor();
-  QString useText( const Object& o, const Objects& sel, const KigDocument& d,
-                   const KigWidget& v ) const;
 };
 
 class GenericIntersectionConstructor
@@ -100,6 +95,24 @@ public:
   ~GenericIntersectionConstructor();
 
   bool isIntersection() const;
+
+  QString useText( const Object& o, const Objects& sel, const KigDocument& d,
+                   const KigWidget& v ) const;
+};
+
+class MidPointOfTwoPointsConstructor
+  : public StandardConstructorBase
+{
+  ArgParser mparser;
+public:
+  MidPointOfTwoPointsConstructor();
+  ~MidPointOfTwoPointsConstructor();
+  void drawprelim( KigPainter& p, const Objects& parents,
+                   const KigDocument& ) const;
+  Objects build( const Objects& os, KigDocument& d,
+                 KigWidget& w ) const;
+  void plug( KigDocument* doc, KigGUIAction* kact );
+  bool isTransform() const;
 };
 
 #endif
