@@ -23,12 +23,15 @@
 
 #include <kdebug.h>
 #include <klineedit.h>
+#include <kapplication.h>
 
 MacroWizard::MacroWizard( QWidget* parent, DefineMacroMode* m )
   : MacroWizardBase( parent, "Define Macro Wizard", false ), mmode( m )
 {
   connect( KLineEdit2, SIGNAL( textChanged( const QString& ) ),
            this, SLOT( nameTextChanged( const QString& ) ) );
+  connect( this, SIGNAL( helpClicked() ), this,
+           SLOT( slotHelpClicked() ) );
 };
 
 MacroWizard::~MacroWizard()
@@ -80,3 +83,9 @@ void MacroWizard::accept()
 {
   mmode->finishPressed();
 }
+
+void MacroWizard::slotHelpClicked()
+{
+  kapp->invokeHelp( QString::fromLatin1( "defining-macros"),
+                    QString::fromLatin1( "kig" ) );
+}
