@@ -23,6 +23,7 @@
 
 #include "../kig/kig_part.h"
 #include "../kig/kig_view.h"
+#include "../kig/kig_commands.h"
 #include "../misc/i18n.h"
 #include "../objects/object_imp.h"
 #include "../objects/object.h"
@@ -30,6 +31,7 @@
 #include "../objects/object_factory.h"
 #include "../misc/lists.h"
 #include "../misc/argsparser.h"
+#include "../misc/coordinate_system.h"
 #include "../misc/object_constructor.h"
 #include "construct_mode.h"
 #include "normal.h"
@@ -38,6 +40,7 @@
 #include <qcursor.h>
 #include <qpainter.h>
 #include <qpen.h>
+#include <qdialog.h>
 #include <kglobal.h>
 #include <kiconloader.h>
 
@@ -75,7 +78,7 @@ public:
                               KigDocument& doc, KigWidget& w, NormalMode& m ) = 0;
 };
 
-class BuiltinActionsProvider
+class BuiltinObjectActionsProvider
   : public PopupActionProvider
 {
 public:
@@ -85,10 +88,22 @@ public:
                       KigDocument& doc, KigWidget& w, NormalMode& m );
 };
 
+class BuiltinDocumentActionsProvider
+  : public PopupActionProvider
+{
+  int mnumberofcoordsystems;
+  bool misfullscreen;
+public:
+  void fillUpMenu( NormalModePopupObjects& popup, int menu, int& nextfree );
+  bool executeAction( int menu, int& id, const Objects& os,
+                      NormalModePopupObjects& popup,
+                      KigDocument& doc, KigWidget& w, NormalMode& m );
+};
+
 class ObjectConstructorActionsProvider
   : public PopupActionProvider
 {
-  std::vector<ObjectConstructor*> mctors[7];
+  std::vector<ObjectConstructor*> mctors[NormalModePopupObjects::NumberOfMenus];
 public:
   void fillUpMenu( NormalModePopupObjects& popup, int menu, int& nextfree );
   bool executeAction( int menu, int& id, const Objects& os,
@@ -125,25 +140,35 @@ NormalModePopupObjects::NormalModePopupObjects( KigDocument& doc,
   : KPopupMenu( &view ), mplc( QCursor::pos() ), mdoc( doc ), mview( view ), mobjs( objs ),
     mmode( mode )
 {
-  assert ( ! objs.empty() );
+  bool empty = objs.empty();
   bool single = objs.size() == 1;
   connect( this, SIGNAL( activated( int ) ), this, SLOT( toplevelMenuSlot( int ) ) );
 
-  insertTitle( single ? ObjectImp::translatedName( objs[0]->imp()->id() )
+  insertTitle( empty ? i18n( "Kig Document" )
+               : single ? ObjectImp::translatedName( objs[0]->imp()->id() )
                : i18n( "%1 Objects" ).arg( objs.size() ), 1 );
 
+  if ( empty )
+  {
+    // provides some diverse stuff like "unhide all", set coordinate
+    // system etc.
+    mproviders.push_back( new BuiltinDocumentActionsProvider() );
+  };
   // construct an object using these objects and start constructing an
   // object using these objects
   mproviders.push_back( new ObjectConstructorActionsProvider() );
-  // stuff like hide, show, delete, set size, set color..
-  mproviders.push_back( new BuiltinActionsProvider() );
-  // show property as text label -> show menu
-  // and construct property's as objects -> construct menu
-  mproviders.push_back( new PropertiesActionsProvider() );
-  // stuff like "redefine point" for a fixed or constrained point..
-  mproviders.push_back( new ObjectTypeActionsProvider() );
+  if ( ! empty )
+  {
+    // stuff like hide, show, delete, set size, set color..
+    mproviders.push_back( new BuiltinObjectActionsProvider() );
+    // show property as text label -> show menu
+    // and construct property's as objects -> construct menu
+    mproviders.push_back( new PropertiesActionsProvider() );
+    // stuff like "redefine point" for a fixed or constrained point..
+    mproviders.push_back( new ObjectTypeActionsProvider() );
+  }
 
-  for ( int i = 0; i < 6; ++i )
+  for ( int i = 0; i < (int) NumberOfMenus; ++i )
     mmenus[i] = new QPopupMenu( this );
 
   connect( mmenus[TransformMenu], SIGNAL( activated( int ) ),
@@ -158,24 +183,28 @@ NormalModePopupObjects::NormalModePopupObjects( KigDocument& doc,
            this, SLOT( setColorMenuSlot( int ) ) );
   connect( mmenus[SetSizeMenu], SIGNAL( activated( int ) ),
            this, SLOT( setSizeMenuSlot( int ) ) );
+  connect( mmenus[SetCoordinateSystemMenu], SIGNAL( activated( int ) ),
+           this, SLOT( setCoordinateSystemMenuSlot( int ) ) );
 
-  for ( int i = TransformMenu; i <= ToplevelMenu; ++i )
+  for ( int i = 0; i <= NumberOfMenus; ++i )
   {
     int nextfree = 10;
     for ( uint j = 0; j < mproviders.size(); ++j )
       mproviders[j]->fillUpMenu( *this, i, nextfree );
   };
-  static const QString menunames[6] =
+  static const QString menunames[NumberOfMenus] =
     {
-      i18n( "Transform" ),
-      i18n( "Construct" ),
-      i18n( "Start" ),
-      i18n( "Show" ),
-      i18n( "Set Color" ),
-      i18n( "Set Size" )
+      i18n( "&Transform" ),
+      i18n( "Const&ruct" ),
+      i18n( "&Start" ),
+      i18n( "Sho&w" ),
+      i18n( "Set Co&lor" ),
+      i18n( "Set Si&ze" ),
+      QString::null,
+      i18n( "Set Coordinate S&ystem" )
     };
   int index = 1;
-  for ( int i = 0; i < 6; ++i )
+  for ( int i = 0; i < NumberOfMenus; ++i )
   {
     if ( mmenus[i]->count() == 0 ) continue;
     insertItem( menunames[i], mmenus[i], i, index++ );
@@ -235,7 +264,7 @@ static const QColor* colors[numberofcolors] =
   &Qt::darkRed
 };
 
-void BuiltinActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu, int& nextfree )
+void BuiltinObjectActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu, int& nextfree )
 {
   if ( menu == NormalModePopupObjects::ToplevelMenu )
   {
@@ -292,7 +321,7 @@ void BuiltinActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu
   }
 }
 
-bool BuiltinActionsProvider::executeAction(
+bool BuiltinObjectActionsProvider::executeAction(
   int menu, int& id, const Objects& os, NormalModePopupObjects& popup,
   KigDocument& doc, KigWidget& w, NormalMode& mode )
 {
@@ -309,7 +338,7 @@ bool BuiltinActionsProvider::executeAction(
       // hide the objects..
       for_each( os.begin(), os.end(),
                 bind2nd( mem_fun( &Object::setShown ), false ) );
-      doc.mode()->objectsRemoved();
+      doc.mode()->redrawScreen();
       break;
     case 1:
     {
@@ -374,17 +403,30 @@ void ObjectConstructorActionsProvider::fillUpMenu( NormalModePopupObjects& popup
 
   for ( vectype::iterator i = vec.begin(); i != vec.end(); ++i )
   {
-    int ret = (*i)->wantArgs( popup.objects(), d, v );
-    if ( ret == ArgsChecker::Invalid ) continue;
     bool add = false;
-    if ( (*i)->isTransform() && popup.objects().size() == 1 ) add = menu == NormalModePopupObjects::TransformMenu;
-    else if ( ( *i )->isIntersection() ) add = menu == NormalModePopupObjects::ToplevelMenu;
-    else if ( ret == ArgsChecker::Complete ) add = menu == NormalModePopupObjects::ConstructMenu;
-    else add = menu == NormalModePopupObjects::StartMenu;
+    if ( popup.objects().empty() )
+    {
+      add = menu == NormalModePopupObjects::StartMenu && ! (*i)->isTransform();
+    }
+    else
+    {
+      int ret = (*i)->wantArgs( popup.objects(), d, v );
+      if ( ret == ArgsChecker::Invalid ) continue;
+      if ( (*i)->isTransform() && popup.objects().size() == 1 ) add = menu == NormalModePopupObjects::TransformMenu;
+      else if ( ( *i )->isIntersection() ) add = menu == NormalModePopupObjects::ToplevelMenu;
+      else if ( ret == ArgsChecker::Complete ) add = menu == NormalModePopupObjects::ConstructMenu;
+      else add = menu == NormalModePopupObjects::StartMenu;
+    };
     if ( add )
     {
-      QPixmap icon = KGlobal::iconLoader()->loadIcon( (*i)->iconFileName(), KIcon::User );
-      popup.addAction( menu, icon, (*i)->descriptiveName(), nextfree++ );
+      QCString iconfile = (*i)->iconFileName();
+      if ( !iconfile.isEmpty() && !iconfile.isNull() )
+      {
+        QPixmap icon = KGlobal::iconLoader()->loadIcon( iconfile, KIcon::User );
+        popup.addAction( menu, icon, (*i)->descriptiveName(), nextfree++ );
+      }
+      else
+        popup.addAction( menu, (*i)->descriptiveName(), nextfree++ );
       mctors[menu].push_back( *i );
     }
   };
@@ -402,7 +444,7 @@ bool ObjectConstructorActionsProvider::executeAction(
   }
 
   ObjectConstructor* ctor = mctors[menu][id];
-  if ( ctor->wantArgs( os, doc, w ) == ArgsChecker::Complete )
+  if ( ! os.empty() && ctor->wantArgs( os, doc, w ) == ArgsChecker::Complete )
   {
     ctor->handleArgs( os, doc, w );
     m.clearSelection();
@@ -436,6 +478,11 @@ void NormalModePopupObjects::setSizeMenuSlot( int i )
   activateAction( SetSizeMenu, i );
 }
 
+void NormalModePopupObjects::setCoordinateSystemMenuSlot( int i )
+{
+  activateAction( SetCoordinateSystemMenu, i );
+}
+
 void NormalModePopupObjects::addAction( int menu, const QPixmap& icon, const QString& name, int id )
 {
   QPopupMenu* m = 0;
@@ -469,13 +516,33 @@ void PropertiesActionsProvider::fillUpMenu( NormalModePopupObjects& popup,
   for ( uint i = 0; i < np; ++i )
   {
     ObjectImp* prop = o->property( i, popup.document() );
-    if ( ( menu == NormalModePopupObjects::ConstructMenu &&
-           prop->inherits( ObjectImp::ID_PointImp ) &&
-           ! o->hasimp( ObjectImp::ID_PointImp ) ) ||
-         ( menu == NormalModePopupObjects::ShowMenu ) )
+    const char* iconfile = o->iconForProperty( i );
+    bool add = true;
+    if ( menu == NormalModePopupObjects::ConstructMenu )
     {
-      popup.addAction( menu, i18n( o->properties()[i] ), nextfree++ );
-      mprops[menu -1].push_back( i );
+      // we don't want imp's like DoubleImp, since we can't show them
+      // anyway..
+      add &= ! prop->inherits( ObjectImp::ID_BogusImp );
+      // we don't want to construct PointImp's coordinate property,
+      // since it would construct a point at the same place as its
+      // parent..
+      add &= ! ( o->hasimp( ObjectImp::ID_PointImp ) &&
+                 prop->inherits( ObjectImp::ID_PointImp ) );
+    }
+    else if ( menu == NormalModePopupObjects::ShowMenu )
+      add &= prop->canFillInNextEscape();
+    if ( add )
+    {
+      if ( iconfile && *iconfile )
+      {
+        QPixmap pix = KGlobal::iconLoader()->loadIcon( iconfile, KIcon::User );
+        popup.addAction( menu, pix, i18n( o->properties()[i] ), nextfree++ );
+      }
+      else
+      {
+        popup.addAction( menu, i18n( o->properties()[i] ), nextfree++ );
+      };
+      mprops[menu-1].push_back( i );
     };
     delete prop;
   };
@@ -514,6 +581,7 @@ bool PropertiesActionsProvider::executeAction(
     Objects ret;
     ret.push_back( new PropertyObject( parent, propid ) );
     ret.push_back( new RealObject( CopyObjectType::instance(), ret ) );
+    ret.calc( doc );
     doc.addObjects( ret );
   };
   return true;
@@ -552,3 +620,79 @@ bool ObjectTypeActionsProvider::executeAction(
   return true;
 }
 
+void BuiltinDocumentActionsProvider::fillUpMenu( NormalModePopupObjects& popup, int menu, int& nextfree )
+{
+  if ( menu == NormalModePopupObjects::ToplevelMenu )
+  {
+    popup.addAction( menu, i18n( "Unhide &All" ), nextfree++ );
+    KIconLoader* l = KGlobal::iconLoader();
+    QPixmap p = l->loadIcon( "viewmag+", KIcon::Toolbar );
+    popup.addAction( menu, p, i18n( "Zoom &In" ), nextfree++ );
+    p = l->loadIcon( "viewmag-", KIcon::Toolbar );
+    popup.addAction( menu, p, i18n( "Zoom &Out" ), nextfree++ );
+    if ( popup.widget().isFullScreen() )
+      popup.addAction( menu, i18n( "E&xit Full Screen Mode" ), nextfree++ );
+  }
+  else if ( menu == NormalModePopupObjects::SetCoordinateSystemMenu )
+  {
+    QStringList l = CoordinateSystemFactory::names();
+    mnumberofcoordsystems = l.count();
+    for ( uint i = 0; i < l.count(); ++i )
+      popup.addAction( menu, l[i], nextfree++ );
+  }
+}
+
+bool BuiltinDocumentActionsProvider::executeAction(
+  int menu, int& id, const Objects&,
+  NormalModePopupObjects& popup,
+  KigDocument& doc, KigWidget& w, NormalMode& m )
+{
+  if ( menu == NormalModePopupObjects::ToplevelMenu )
+  {
+    if ( id == 0 )
+    {
+      doc.showHidden();
+      m.clearSelection();
+      w.redrawScreen();
+      return true;
+    }
+    else if ( id == 1 )
+    {
+      w.zoomIn();
+      return true;
+    }
+    else if ( id == 2 )
+    {
+      w.zoomOut();
+      return true;
+    }
+    else if ( popup.widget().isFullScreen() )
+    {
+      if ( id == 3 )
+      {
+        assert( w.view()->parent()->inherits( "QDialog" ) );
+        static_cast<QDialog*>( w.view()->parent() )->close();
+        return true;
+      }
+      else
+        id -= 1;
+    };
+    id -= 3;
+    return false;
+  }
+  else if ( menu == NormalModePopupObjects::SetCoordinateSystemMenu )
+  {
+    if ( id >= mnumberofcoordsystems )
+    {
+      id -= mnumberofcoordsystems;
+      return false;
+    };
+    CoordinateSystem* sys = CoordinateSystemFactory::build( id );
+    assert( sys );
+    doc.history()->addCommand( KigCommand::changeCoordSystemCommand( doc, sys ) );
+    m.clearSelection();
+    return true;
+  }
+  else return false;
+}
+
