@@ -61,6 +61,12 @@ ObjectImp* BogusImp::property( uint, const KigDocument& ) const
   return new InvalidImp;
 }
 
+const char* BogusImp::iconForProperty( uint ) const
+{
+  assert( false );
+  return "";
+}
+
 DoubleImp::DoubleImp( const double d )
   : mdata( d )
 {
@@ -275,5 +281,63 @@ void TransformationImp::visit( ObjectImpVisitor* vtor ) const
   vtor->visit( this );
 }
 
+bool InvalidImp::equals( const ObjectImp& rhs ) const
+{
+  return !rhs.valid();
+}
 
+bool DoubleImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_DoubleImp ) &&
+    static_cast<const DoubleImp&>( rhs ).data() == mdata;
+}
+
+bool IntImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_IntImp ) &&
+    static_cast<const IntImp&>( rhs ).data() == mdata;
+}
+
+bool StringImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_StringImp ) &&
+    static_cast<const StringImp&>( rhs ).data() == mdata;
+}
+
+bool HierarchyImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_HierarchyImp ) &&
+    static_cast<const HierarchyImp&>( rhs ).data() == mdata;
+}
+
+bool TransformationImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_TransformationImp ) &&
+    static_cast<const TransformationImp&>( rhs ).data() == mdata;
+}
+
+bool InvalidImp::canFillInNextEscape() const
+{
+  return true;
+}
+
+bool DoubleImp::canFillInNextEscape() const
+{
+  return true;
+}
+
+bool IntImp::canFillInNextEscape() const
+{
+  return true;
+}
+
+bool StringImp::canFillInNextEscape() const
+{
+  return true;
+}
+
+bool BogusImp::inherits( int type ) const
+{
+  return type == ID_BogusImp ? true : Parent::inherits( type );
+}
 
