@@ -44,6 +44,7 @@ public:
   const QCStringList properties() const;
   const QCStringList propertiesInternalNames() const;
   ObjectImp* property( uint which, const KigDocument& d ) const;
+  const char* iconForProperty( uint which ) const;
   int impRequirementForProperty( uint which ) const;
 
   bool inherits( int typeID ) const;
@@ -54,6 +55,8 @@ public:
 
   // returns 0 for segment, 1 for ray, 2 for line...
   virtual int type() const = 0;
+
+  bool equals( const ObjectImp& rhs ) const;
 };
 
 class SegmentImp
@@ -81,6 +84,7 @@ public:
   const QCStringList properties() const;
   const QCStringList propertiesInternalNames() const;
   ObjectImp* property( uint which, const KigDocument& d ) const;
+  const char* iconForProperty( uint which ) const;
   int impRequirementForProperty( uint which ) const;
 
   SegmentImp* copy() const;
