@@ -19,6 +19,8 @@
 #include "other_imp.h"
 
 #include "bogus_imp.h"
+#include "point_imp.h"
+#include "line_imp.h"
 
 #include "../misc/screeninfo.h"
 #include "../misc/common.h"
@@ -75,7 +77,7 @@ bool AngleImp::inRect( const Rect& r, int width, const KigWidget& w ) const
 
 const uint AngleImp::numberOfProperties() const
 {
-  return Parent::numberOfProperties() + 2;
+  return Parent::numberOfProperties() + 3;
 }
 
 const QCStringList AngleImp::propertiesInternalNames() const
@@ -83,6 +85,7 @@ const QCStringList AngleImp::propertiesInternalNames() const
   QCStringList l = Parent::propertiesInternalNames();
   l << I18N_NOOP( "angle-radian" );
   l << I18N_NOOP( "angle-degrees" );
+  l << I18N_NOOP( "angle-bisector" );
   assert( l.size() == AngleImp::numberOfProperties() );
   return l;
 }
@@ -92,6 +95,7 @@ const QCStringList AngleImp::properties() const
   QCStringList l = Parent::properties();
   l << I18N_NOOP( "Angle in Radians" );
   l << I18N_NOOP( "Angle in Degrees" );
+  l << I18N_NOOP( "Angle Bisector" );
   assert( l.size() == AngleImp::numberOfProperties() );
   return l;
 }
@@ -103,6 +107,19 @@ int AngleImp::impRequirementForProperty( uint which ) const
   else return ID_AngleImp;
 }
 
+const char* AngleImp::iconForProperty( uint which ) const
+{
+  if ( which < Parent::numberOfProperties() )
+    return Parent::iconForProperty( which );
+  if ( which == Parent::numberOfProperties() )
+    return "angle-size"; // size in radians
+  else if ( which == Parent::numberOfProperties() + 1 )
+    return "angle-size"; // size in degrees
+  else if ( which == Parent::numberOfProperties() + 2 )
+    return "angle-bisector"; // angle bisector..
+  else assert( false );
+}
+
 ObjectImp* AngleImp::property( uint which, const KigDocument& w ) const
 {
   if ( which < Parent::numberOfProperties() )
@@ -111,6 +128,12 @@ ObjectImp* AngleImp::property( uint which, const KigDocument& w ) const
     return new DoubleImp( size() );
   else if ( which == Parent::numberOfProperties() + 1 )
     return new DoubleImp( size() * 180 / M_PI );
+  else if ( which == Parent::numberOfProperties() + 2 )
+  {
+    const double angle = mstartangle + mangle / 2;
+    Coordinate p2 = mpoint + Coordinate( cos( angle ), sin( angle ) ) * 10;
+    return new RayImp( mpoint, p2 );
+  }
   else assert( false );
 }
 
@@ -182,6 +205,11 @@ int VectorImp::impRequirementForProperty( uint which ) const
   return Parent::impRequirementForProperty( which );
 }
 
+const char* VectorImp::iconForProperty( uint which ) const
+{
+  return Parent::iconForProperty( which );
+}
+
 ObjectImp* VectorImp::property( uint which, const KigDocument& w ) const
 {
   return Parent::property( which, w );
@@ -254,10 +282,31 @@ ArcImp* ArcImp::copy() const
   return new ArcImp( mcenter, mradius, msa, ma );
 }
 
-ObjectImp* ArcImp::transform( const Transformation& ) const
+ObjectImp* ArcImp::transform( const Transformation& t ) const
 {
-  // TODO
-  return new InvalidImp();
+  //
+  // we don't have conic arcs! So it is invalid to transform an arc
+  // with a nonhomothetic transformation
+  //
+  if ( ! t.isHomothetic() ) return new InvalidImp();
+
+  bool valid = true;
+  Coordinate nc = t.apply( mcenter, valid );
+  double nr = t.apply( mradius );
+  // transform msa...
+  double nmsa = msa;
+  if ( t.getAffineDeterminant() > 0 )
+  {
+    nmsa = msa - t.getRotationAngle();
+  } else
+  {
+    Coordinate ar = t.apply2by2only( Coordinate( cos(msa), sin(msa) ) );
+    nmsa = atan2( ar.y, ar.x );
+    nmsa -= ma;
+  }
+  while ( nmsa < -M_PI ) nmsa += 2*M_PI;
+  while ( nmsa > M_PI ) nmsa -= 2*M_PI;
+  return new ArcImp( nc, nr, nmsa, ma );
 }
 
 void ArcImp::draw( KigPainter& p ) const
@@ -289,27 +338,78 @@ bool ArcImp::valid() const
 
 const uint ArcImp::numberOfProperties() const
 {
-  return Parent::numberOfProperties();
+  return Parent::numberOfProperties() + 6;
 }
 
 const QCStringList ArcImp::properties() const
 {
-  return Parent::properties();
+  QCStringList ret = Parent::properties();
+  ret << I18N_NOOP( "Center" );
+  ret << I18N_NOOP( "Radius" );
+  ret << I18N_NOOP( "Angle in Degrees" );
+  ret << I18N_NOOP( "Angle in Radians" );
+  ret << I18N_NOOP( "Sector Surface" );
+  ret << I18N_NOOP( "Arc Length" );
+  assert( ret.size() == ArcImp::numberOfProperties() );
+  return ret;
 }
 
 const QCStringList ArcImp::propertiesInternalNames() const
 {
-  return Parent::propertiesInternalNames();
+  QCStringList ret = Parent::propertiesInternalNames();
+  ret << "center";
+  ret << "radius";
+  ret << "angle-degrees";
+  ret << "angle-radians";
+  ret << "sector-surface";
+  ret << "arc-length";
+  return ret;
+}
+
+const char* ArcImp::iconForProperty( uint which ) const
+{
+  if ( which < Parent::numberOfProperties() )
+    return Parent::iconForProperty( which );
+  else if ( which == Parent::numberOfProperties() )
+    return "";
+  else if ( which == Parent::numberOfProperties() + 1 )
+    return "";
+  else if ( which == Parent::numberOfProperties() + 2 )
+    return "angle-size";
+  else if ( which == Parent::numberOfProperties() + 3 )
+    return "angle-size";
+  else if ( which == Parent::numberOfProperties() + 4 )
+    return "";
+  else if ( which == Parent::numberOfProperties() + 5 )
+    return "";
+  else assert( false );
 }
 
 ObjectImp* ArcImp::property( uint which, const KigDocument& d ) const
 {
-  return Parent::property( which, d );
+  if ( which < Parent::numberOfProperties() )
+    return Parent::property( which, d );
+  else if ( which == Parent::numberOfProperties() )
+    return new PointImp( mcenter );
+  else if ( which == Parent::numberOfProperties() + 1 )
+    return new DoubleImp( mradius );
+  else if ( which == Parent::numberOfProperties() + 2 )
+    return new IntImp( static_cast<int>( ma * 180 / M_PI ) );
+  else if ( which == Parent::numberOfProperties() + 3 )
+    return new DoubleImp( ma );
+  else if ( which == Parent::numberOfProperties() + 4 )
+    return new DoubleImp( mradius * mradius * ma / 2 );
+  else if ( which == Parent::numberOfProperties() + 5 )
+    return new DoubleImp( mradius * ma );
+  else assert( false );
 }
 
 int ArcImp::impRequirementForProperty( uint which ) const
 {
-  return Parent::impRequirementForProperty( which );
+  if ( which < Parent::numberOfProperties() )
+    return Parent::impRequirementForProperty( which );
+  else
+    return ID_ArcImp;
 }
 
 bool ArcImp::inherits( int type ) const
@@ -337,6 +437,10 @@ double ArcImp::getParam( const Coordinate& c, const KigDocument& ) const
   Coordinate d = (c - mcenter).normalize();
   double angle = atan2( d.y, d.x );
   angle -= msa;
+// mp: problems with large arcs
+  while ( angle > ma/2 + M_PI ) angle -= 2*M_PI;
+  while ( angle < ma/2 - M_PI ) angle += 2*M_PI;
+//
   angle = max( 0., min( angle, ma ) );
   angle /= ma;
   return angle;
@@ -379,3 +483,26 @@ const Coordinate VectorImp::b() const
 {
   return mb;
 }
+
+bool ArcImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_ArcImp ) &&
+    static_cast<const ArcImp&>( rhs ).radius() == radius() &&
+    static_cast<const ArcImp&>( rhs ).startAngle() == startAngle() &&
+    static_cast<const ArcImp&>( rhs ).angle() == angle();
+}
+
+bool AngleImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_AngleImp ) &&
+    static_cast<const AngleImp&>( rhs ).point() == point() &&
+    static_cast<const AngleImp&>( rhs ).startAngle() == startAngle() &&
+    static_cast<const AngleImp&>( rhs ).angle() == angle();
+}
+
+bool VectorImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_VectorImp ) &&
+    static_cast<const VectorImp&>( rhs ).a() == a() &&
+    static_cast<const VectorImp&>( rhs ).b() == b();
+}
