@@ -25,11 +25,19 @@
 #include "object.h"
 
 #include "../misc/common.h"
+#include "../misc/calcpaths.h"
+#include "../kig/kig_part.h"
+#include "../kig/kig_view.h"
+#include "../kig/kig_commands.h"
 
 #include <functional>
 #include <algorithm>
 #include <cmath>
 
+using std::find;
+
+#include <qstringlist.h>
+
 static const char* constructanglethroughpoint =
   I18N_NOOP( "Construct an angle through this point" );
 
@@ -68,8 +76,6 @@ ObjectImp* AngleType::calc( const Args& parents, const KigDocument& ) const
 
   Coordinate lvect = points[0] - points[1];
   Coordinate rvect = points[2] - points[1];
-  lvect = lvect.normalize();
-  rvect = rvect.normalize();
 
   double startangle = atan2( lvect.y, lvect.x );
   double endangle = atan2( rvect.y, rvect.x );
@@ -106,14 +112,14 @@ ObjectImp* VectorType::calc( const Coordinate& a, const Coordinate& b ) const
   return new VectorImp( a, b );
 }
 
-static const struct ArgParser::spec argsspec1c[] =
+static const struct ArgParser::spec argsspecLocus[] =
 {
   { ObjectImp::ID_HierarchyImp, "hierarchy" },
   { ObjectImp::ID_CurveImp, "curve" }
 };
 
 LocusType::LocusType()
-  : ArgparserObjectType( "Locus", argsspec1c, 2 )
+  : ArgparserObjectType( "Locus", argsspecLocus, 2 )
 {
 }
 
@@ -198,12 +204,10 @@ int CopyObjectType::resultId() const
 int LocusType::impRequirement( const ObjectImp* o, const Args& parents ) const
 {
   Args firsttwo( parents.begin(), parents.begin() + 2 );
-  if ( margsparser.check( firsttwo ) == ArgsChecker::Invalid )
-  {
-    return o->inherits( ObjectImp::ID_HierarchyImp ) ? ObjectImp::ID_HierarchyImp
-      : ObjectImp::ID_CurveImp;
-  }
-  else return ObjectImp::ID_AnyImp;
+  if ( find( firsttwo.begin(), firsttwo.end(), o ) != firsttwo.end() )
+    return margsparser.impRequirement( o, firsttwo );
+  else
+    return ObjectImp::ID_AnyImp;
 }
 
 const LocusType* LocusType::instance()
@@ -212,7 +216,7 @@ const LocusType* LocusType::instance()
   return &t;
 }
 
-static const ArgParser::spec argsspec3p[] =
+static const ArgParser::spec argsspecArcBTP[] =
 {
   { ObjectImp::ID_PointImp, I18N_NOOP( "Construct an arc starting at this point" ) },
   { ObjectImp::ID_PointImp, I18N_NOOP( "Construct an arc through this point" ) },
@@ -220,7 +224,7 @@ static const ArgParser::spec argsspec3p[] =
 };
 
 ArcBTPType::ArcBTPType()
-  : ArgparserObjectType( "ArcBTP", argsspec3p, 3 )
+  : ArgparserObjectType( "ArcBTP", argsspecArcBTP, 3 )
 {
 }
 
@@ -252,6 +256,7 @@ ObjectImp* ArcBTPType::calc( const Args& args, const KigDocument& ) const
     assert( args[2]->inherits( ObjectImp::ID_PointImp ) );
     Coordinate c = static_cast<const PointImp*>( args[2] )->coordinate();
     center = calcCenter( a, b, c );
+    if ( ! center.valid() ) return new InvalidImp;
     Coordinate ad = a - center;
     Coordinate bd = b - center;
     Coordinate cd = c - center;
@@ -308,5 +313,58 @@ int ArcBTPType::resultId() const
   return ObjectImp::ID_ArcImp;
 }
 
+QStringList AngleType::specialActions() const
+{
+  QStringList ret;
+  ret << i18n( "Set Si&ze" );
+  return ret;
+}
 
+void AngleType::executeAction(
+  int i, RealObject* o, KigDocument& d, KigWidget& w,
+  NormalMode& ) const
+{
+  assert( i == 0 );
 
+  Objects parents = o->parents();
+  assert( parents.size() == 3 );
+
+  if ( ! parents[0]->hasimp( ObjectImp::ID_PointImp ) ||
+       ! parents[1]->hasimp( ObjectImp::ID_PointImp ) ||
+       ! parents[2]->hasimp( ObjectImp::ID_PointImp ) )
+    return;
+
+  Coordinate a = static_cast<const PointImp*>( parents[0]->imp() )->coordinate();
+  Coordinate b = static_cast<const PointImp*>( parents[1]->imp() )->coordinate();
+  Coordinate c = static_cast<const PointImp*>( parents[2]->imp() )->coordinate();
+
+  Coordinate lvect = a - b;
+  Coordinate rvect = c - b;
+
+  double startangle = atan2( lvect.y, lvect.x );
+  double endangle = atan2( rvect.y, rvect.x );
+  double anglelength = endangle - startangle;
+  if ( anglelength < 0 ) anglelength += 2* M_PI;
+  if ( startangle < 0 ) startangle += 2*M_PI;
+
+  int anglelengthdeg = static_cast<int>( anglelength * 180 / M_PI );
+
+  bool ok = true;
+  double newsize = getDoubleFromUser(
+    i18n( "Set Angle Size" ), i18n( "Choose the new size: " ),
+    anglelengthdeg, &w, &ok, -2147483647, 2147483647, 0 );
+  if ( ! ok ) return;
+
+  newsize *= M_PI;
+  newsize /= 180;
+
+  double newcangle = startangle + newsize;
+  Coordinate cdir( cos( newcangle ), sin( newcangle ) );
+  Coordinate nc = b + cdir.normalize( rvect.length() );
+
+  MonitorDataObjects mon( getAllParents( parents ) );
+  parents[2]->move( c, nc - c, d );
+  KigCommand* kc = new KigCommand( d, i18n( "Resize &Angle" ) );
+  kc->addTask( mon.finish() );
+  d.history()->addCommand( kc );
+}
