@@ -56,7 +56,7 @@ const uint PointImp::numberOfProperties() const
 const QCStringList PointImp::propertiesInternalNames() const
 {
   QCStringList l = Parent::propertiesInternalNames();
-  l << I18N_NOOP( "coordinate" );
+  l << "coordinate";
   assert( l.size() == PointImp::numberOfProperties() );
   return l;
 }
@@ -76,6 +76,15 @@ int PointImp::impRequirementForProperty( uint which ) const
   else return ID_PointImp;
 }
 
+const char* PointImp::iconForProperty( uint which ) const
+{
+  if ( which < Parent::numberOfProperties() )
+    return Parent::iconForProperty( which );
+  if ( which == Parent::numberOfProperties() )
+    return "pointxy"; // coordinate
+  else assert( false );
+}
+
 ObjectImp* PointImp::property( uint which, const KigDocument& d ) const
 {
   if ( which < Parent::numberOfProperties() )
@@ -132,4 +141,13 @@ void PointImp::visit( ObjectImpVisitor* vtor ) const
   vtor->visit( this );
 }
 
+bool PointImp::equals( const ObjectImp& rhs ) const
+{
+  return rhs.inherits( ID_PointImp ) &&
+    static_cast<const PointImp&>( rhs ).coordinate() == coordinate();
+}
 
+bool PointImp::canFillInNextEscape() const
+{
+  return true;
+}
