@@ -50,6 +50,7 @@ public:
   const QCStringList properties() const;
   const QCStringList propertiesInternalNames() const;
   ObjectImp* property( uint which, const KigDocument& w ) const;
+  const char* iconForProperty( uint which ) const;
   int impRequirementForProperty( uint which ) const;
 
   bool inherits( int typeID ) const;
@@ -61,6 +62,9 @@ public:
 
   QString text() const;
   const Coordinate coordinate() const;
+  bool hasFrame() const;
+
+  bool equals( const ObjectImp& rhs ) const;
 };
 
 #endif
